package types

type GiftCenterItem struct {
	ID        int    `json:"page_id"`
	Name      string `json:"page_name"`
	Type      int    `json:"gift_type"`
	Title     string `json:"page_content"`
	St        string `json:"page_starttime"`
	Et        string `json:"page_endtime"`
	Level     int    `json:"level_limit"`
	StartTime int64  `json:"-"`
	EndTime   int64  `json:"-"`
}

type GiftItem struct {
	ID         int64     `json:"gift_id,omitempty"`
	Name       string    `json:"gift_name,omitempty"`
	Price      int       `json:"gift_price,omitempty"`
	Materials  [][]int64 `json:"gift_award,omitempty"`
	Limit      int       `json:"gift_limit,omitempty"`
	StartTime  string    `json:"gift_start_time,omitempty"`
	EndTime    string    `json:"gift_end_time,omitempty"`
	Show       int       `json:"show_or_not,omitempty"`
	Type       int       `json:"type,omitempty"`
	Level      int       `json:"level_limit,omitempty"`
	AdCoolDown int       `json:"cd,omitempty"` // 广告间隔时间
	MaxLevel   int       `json:"maxLevel,omitempty"`
}

// 日特惠、角色礼包
type MidGiftItem struct {
	ID        int64     `json:"gift_id"`
	Name      string    `json:"gift_name"`
	Price     int       `json:"gift_price"`
	Materials [][]int64 `json:"gift_award"`
	Limit     int       `json:"gift_limit"`
	StartTime string    `json:"gift_start_time"`
	EndTime   string    `json:"gift_end_time"`
	Show      int       `json:"show_or_not"`
	Type      int       `json:"type"`
}

// 新手礼包
type MidNewerItem struct {
	ID        int64     `json:"gift_id"`
	Name      string    `json:"gift_name"`
	Price     int       `json:"gift_price"`
	Materials [][]int64 `json:"gift_award"`
	Limit     int       `json:"gift_limit"`
	Level     int       `json:"level_limit"`
}

// 日特惠、角色礼包
type MidNewYearGiftItem struct {
	ID         int64     `json:"gift_id"`
	Name       string    `json:"gift_name"`
	Type       int       `json:"gift_type"`
	Price      int       `json:"gift_price"`
	Materials  [][]int64 `json:"gift_award"`
	Limit      int       `json:"gift_limit"`
	StartTime  string    `json:"gift_start_time"`
	EndTime    string    `json:"gift_end_time"`
	Show       int       `json:"show_or_not"`
	PriceType  int       `json:"priceType"`
	AdCoolDown int       `json:"cd"` // 广告间隔时间
}

type MidHoliday38Gift struct {
	ID        int64     `json:"gift_id"`
	Name      string    `json:"gift_name"`
	StartTime string    `json:"gift_start_time"`
	EndTime   string    `json:"gift_end_time"`
	Type      int       `json:"gift_type"`
	Price     int       `json:"gift_price"`
	Materials [][]int64 `json:"gift_award"`
	Limit     int       `json:"gift_limit"`
	Show      int       `json:"show_or_not"`
}

type SpecialGift struct {
	ID        int64     `json:"gift_id"`
	Name      string    `json:"gift_name"`
	Price     int       `json:"gift_price"`
	Materials [][]int64 `json:"gift_award1"`
	MaxLevel  int       `json:"maxLevel"`
}
type WeekendGiftAttr struct {
	ID         int64  `json:"ID"`
	Name       string `json:"name"`
	Week       int    `json:"week"`
	Weight     int    `json:"weight"`
	LimitType  int    `json:"type"`  // 限购类型
	LimitCount int    `json:"count"` // 限购次数
	Cost       int    `json:"cost"`  // 价格
}
type WeekendGift struct {
	GiftId        int64   `json:"gift_id"`
	Name          string  `json:"gift_name"`
	Type          int     `json:"type"`
	Sort          int     `json:"sort"`
	Week          int     `json:"week"`
	Weight        int     `json:"weight"`
	PriceType     int64   `json:"priceType"`
	OriginalPrice int     `json:"original_price"`
	GiftPrice     int     `json:"gift_price"`
	Discount      int     `json:"discount"`
	GiftAward     [][]int `json:"gift_award"`
	GiftStartTime string  `json:"gift_start_time"`
	GiftEndTime   string  `json:"gift_end_time"`
	GiftLimitType []int   `json:"gift_limit_type"`
	GiftLimit     int     `json:"gift_limit"`
	ShowOrNot     int     `json:"show_or_not"`
}

type MidBoxKeyGiftItem struct {
	ID         int64     `json:"GiftPackID"` // 礼包id
	Name       string    `json:"PackName"`
	Type       int       `json:"GoodsType"`       // 礼包类型
	BoxType    int       `json:"BoxType"`         // 宝箱类型
	Materials  [][]int64 `json:"PropList"`        // 道具内容
	GetType    int       `json:"GetType"`         // 获取类型 0 免费 1 广告 2 钻石 3 现金充值  广告分为看广告和广告劵
	Price      int       `json:"Price"`           // 价格/数量
	Limit      int       `json:"LimitTimes"`      // 限购次数
	AdCoolDown int       `json:"AdCoolDown"`      // 广告间隔时间
	PlatGroup  int       `json:"platformGroupId"` // 平台
}

type MidEquipBoxBase struct {
	GoodsId int64  `json:"goods_id,omitempty"`
	Name    string `json:"goods_name,omitempty"`
	BoxType int    `json:"BoxType,omitempty"`  // 宝箱类型
	Cost    int64  `json:"OpenProp,omitempty"` // 宝箱开启消耗道具
	// OpenTimes       []int `json:"OpenTimes"`            // 宝箱开启次数
	// SeniorOpenTimes int   `json:"DiamondConvertGrowUp"` // 钻石兑换钥匙的每次成长金额
}
type MidEndlessPassGift struct {
	ID                 int64  `json:"GiftPackID"`
	GoodsType          int    `json:"GoodsType"`
	Name               string `json:"Name"`
	GetType            int    `json:"GetType"`
	Price              int    `json:"Price"`
	RewardNum          int    `json:"RewardNum"`
	ReturnDiamond      int    `json:"ReturnDiamond"`
	AddPassportExp     int    `json:"AddPassportExp"`
	GetPassportExpRate int    `json:"GetPassportExpRate"`
}

type MidSalesSalesGoods struct {
	ID         int64     `json:"goods_id"`
	Name       string    `json:"name"`
	Type       int       `json:"type"`
	AdCoolDown int       `json:"ad_cd"`
	AdDayTimes int       `json:"ad_times"`
	AdDayType  int       `json:"ad_type"`
	Price      [][]int64 `json:"price_sever"`
	Content    [][]int64 `json:"content"`
	GiftNum    []int64   `json:"give"`
	Double     int       `json:"double"`
	PlatGroup  int       `json:"platformGroupId"` // 平台
}

type MidCol struct {
	Key   string `json:"key"`
	Type  string `json:"format"`
	Value string `json:"value"`
}
type MidPropStoreGoods struct {
	ID      int64   `json:"Shop_ID"`
	Name    string  `json:"name,omitempty"`
	Type    int     `json:"Item_Type"`
	Price   []int64 `json:"Price"`
	Content []int64 `json:"Output"`
}
type LimitGiftGoods struct {
	Id    int64     `json:"id"` // 礼包ID
	Name  string    `json:"name"`
	Type  int       `json:"type"`  // 礼包类型
	Award [][]int64 `json:"award"` // 礼包奖励
	Price []int64   `json:"price"` // 价格 [购买方式,对应价格,_原价]
}
