package model

const (
	LotteryTypeHunt       = int64(iota + 1) // 寻宝
	LotteryTypeEggMachine                   // 扭蛋
)

// 抽奖活动列表-LotteryList exported from 活动-循环活动.xlsx
type LotteryList struct {
	Uid          int64       `json:"Uid"`          // ID
	OpenActivity int64       `json:"OpenActivity"` // 是否开服活动 0-不是 1-是
	Type         int64       `json:"Type"`         // 活动类型 1-寻宝 2-扭蛋
	ActivitySet  int64       `json:"ActivitySet"`  // 活动组ID 必填项--关联活动组，进度奖，兑换，抽奖，签到补给表
	Name         string      `json:"Name"`         // 活动名称
	StartTime    string      `json:"StartTime"`    // 开始时间
	EndTime      string      `json:"EndTime"`      // 结束时间
	Background   int64       `json:"Background"`   // 背景图集资源
	ButtonName   []string    `json:"ButtonName"`   // 按钮名称
	Icon         string      `json:"Icon"`         // 入口ICON资源
	Consume      [][]int64   `json:"Consume"`      // 抽奖消耗 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情
	Points       int64       `json:"Points"`       // 积分上限 对应C列活动类型 1=200 2=650
	GiftPack     [][]int64   `json:"GiftPack"`     // 礼包 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情
	Quota        [][]int64   `json:"Quota"`        // 礼包限购
	Price        [][]int64   `json:"Price"`        // 礼包价格
	Discount     [][]float64 `json:"Discount"`     // 礼包折扣值 百分比值
	FundPrice    int64       `json:"FundPrice"`    // 计费点ID
} // package model

// 活动组-LotterySet exported from 活动-循环活动.xlsx
type LotterySet struct {
	Uid         int64     `json:"Uid"`         // ID
	ActivitySet int64     `json:"ActivitySet"` // 活动组ID
	Reward      [][]int64 `json:"Reward"`      // 奖励 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情 因UI限制该奖励数量固定为10个不可多或少
	Weight      int64     `json:"Weight"`      // 权重  中奖概率，每个组必须为10000
	Sign        int64     `json:"Sign"`        // 标识 0-无 1-秘宝 2-稀有 3-大奖
	Sort        int64     `json:"Sort"`        // 排序 从左上角开始1，顺时针排序
} // package model

// 进度奖-LotteryProgress exported from 活动-循环活动.xlsx
type LotteryProgress struct {
	Uid         int64     `json:"Uid"`         // ID
	ActivitySet int64     `json:"ActivitySet"` // 活动组ID
	Num         int64     `json:"Num"`         // 次数 因UI限制奖励固定为8个，不可多不可少
	Reward      [][]int64 `json:"Reward"`      // 奖励 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情
} // package model

// 兑换-LotteryExchange exported from 活动-循环活动.xlsx
type LotteryExchange struct {
	Uid         int64   `json:"Uid"`         // ID
	ActivitySet int64   `json:"ActivitySet"` // 活动组ID
	Quota       int64   `json:"Quota"`       // 限购次数
	GiftPack    []int64 `json:"GiftPack"`    // 礼包ID 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情
	Price       []int64 `json:"Price"`       // 价格 [支付方式,道具ID,数量] 1-现金 2-广告 3-道具 4-免费
	CdrpId      int64   `json:"CdrpId"`      // 计费点
	Video       int64   `json:"Video"`       // 广告点ID
} // package model

// 二级抽奖-LotteryPrizeDraw exported from 活动-循环活动.xlsx
type LotteryPrizeDraw struct {
	Uid         int64     `json:"Uid"`         // ID
	ActivitySet int64     `json:"ActivitySet"` // 活动组ID
	Reward      [][]int64 `json:"Reward"`      // 奖励 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情 因UI限制该奖励数量固定为10个不可多或少
	Weight      int64     `json:"Weight"`      // 权重  中奖概率，每个组必须为10000
} // package model

// 二级抽奖消耗-LotteryPrizeDrawCon exported from 活动-循环活动.xlsx
type LotteryPrizeDrawCon struct {
	Uid         int64     `json:"Uid"`         // ID
	ActivitySet int64     `json:"ActivitySet"` // 活动组ID
	Reward      [][]int64 `json:"Reward"`      // 消耗道具 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情 因UI限制该奖励数量固定为10个不可多或少
} // package model

// 签到补给-LotterySupply exported from 活动-循环活动.xlsx
type LotterySupply struct {
	Uid         int64     `json:"Uid"`         // ID
	ActivitySet int64     `json:"ActivitySet"` // 活动组ID
	Time        int64     `json:"Time"`        // 天数
	Free        [][]int64 `json:"Free"`        // 奖励（类型，道具ID，数量） 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情
	Senior      [][]int64 `json:"Senior"`      // 奖励（类型，道具ID，数量） 1:英雄碎片 2:假道具 3：材料 4：抽奖道具 5：宝物道具 6：宝箱类型 7：英雄皮肤 8：要塞皮肤 9.符文 10：场景皮肤 11：铭文 12:头像道具 13：头像框道具 14：聊天气泡道具 15：游戏内道具 16：体力类型 17：杂类 18：经验，玩家经验、通行证经验 19:聊天动态表情
}
